from typing import Dict, List
import json

from workspaces.schema import Workflow, WorkflowTemplate, WorkflowRequest, WorkflowRequestFile
from workspaces.services import WorkflowService
from workspaces.workflow_interfaces import WorkflowInfoIF, WorkflowIF, WorkflowRequestIF, WorkflowRequestFileIF


class FakeWorkflowInfo(WorkflowInfoIF):
    def __init__(self):
        self.requests = []

    def lookup_workflow_request(self, request_id: int) -> WorkflowIF:
        return self.requests[request_id-1]

    def all_workflows(self) -> List[WorkflowIF]:
        return [self.lookup_workflow_definition('null')]

    def create_workflow_request(self, workflow: WorkflowIF, argument: Dict) -> WorkflowRequestIF:
        request = WorkflowRequest(workflow_name=workflow.workflow_name, argument=argument)
        self.save_request(request)
        return request

    def save_request(self, request: WorkflowRequestIF):
        request.workflow_request_id = len(self.requests)+1
        self.requests.append(request)

    def save_file(self, request: WorkflowRequestIF, filename: str, content: bytes) -> WorkflowRequestFileIF:
        request.files.append(WorkflowRequestFile(filename=filename, content=content))

    def lookup_workflow_definition(self, name: str) -> 'Workflow':
        if name == 'null':
            w = Workflow(workflow_name='null')
            w.templates.append(WorkflowTemplate(filename='null.condor',
                                                content=b'executable = null\narguments = "-n"\n\nqueue'))
            return w
        raise NotImplementedError


info = FakeWorkflowInfo()


def test_workflow_creation():
    # This seems like a bit of a blunt instrument for testing the service. We should probably break the service down
    # into smaller pieces.
    global info
    service = WorkflowService(info)

    # create request
    request = info.create_workflow_request(info.lookup_workflow_definition('null'), json.loads("argument:'-n'"))
    # execute the null workflow
    service.execute(request, [])
